@@ -11,6 +11,7 @@
  * the GNU General Public License
  */
 
+#include <asterisk/lock.h>
 #include <asterisk/file.h>
 #include <asterisk/logger.h>
 #include <asterisk/channel.h>
@@ -21,9 +22,6 @@
 #include <string.h>
 #include <stdlib.h>
 
-#include <pthread.h>
-
-
 static char *tdesc = "Simple Echo Application";
 
 static char *app = "Echo";
@@ -47,10 +45,18 @@ static int echo_exec(struct ast_channel *chan, void *data)
 	ast_set_write_format(chan, ast_best_codec(chan->nativeformats));
 	ast_set_read_format(chan, ast_best_codec(chan->nativeformats));
 	/* Do our thing here */
-	while((f = ast_read(chan))) {
+	while(ast_waitfor(chan, -1) > -1) {
+		f = ast_read(chan);
+		if (!f)
+			break;
+		f->delivery.tv_sec = 0;
+		f->delivery.tv_usec = 0;
 		if (f->frametype == AST_FRAME_VOICE) {
 			if (ast_write(chan, f)) 
 				break;
+		} else if (f->frametype == AST_FRAME_VIDEO) {
+			if (ast_write(chan, f)) 
+				break;
 		} else if (f->frametype == AST_FRAME_DTMF) {
 			if (f->subclass == '#') {
 				res = 0;
