@@ -3,14 +3,15 @@
  *
  * Silly application to play an MP3 file -- uses mpg123
  * 
- * Copyright (C) 1999, Mark Spencer
+ * Copyright (C) 1999-2004, Digium, Inc.
  *
- * Mark Spencer <markster@linux-support.net>
+ * Mark Spencer <markster@digium.com>
  *
  * This program is free software, distributed under the terms of
  * the GNU General Public License
  */
  
+#include <asterisk/lock.h>
 #include <asterisk/file.h>
 #include <asterisk/logger.h>
 #include <asterisk/channel.h>
@@ -24,9 +25,9 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <fcntl.h>
-#include <pthread.h>
 #include <sys/time.h>
 
+#define LOCAL_MPG_123 "/usr/local/bin/mpg123"
 #define MPG_123 "/usr/bin/mpg123"
 
 static char *tdesc = "Silly MP3 Application";
@@ -59,36 +60,57 @@ static int mp3play(char *filename, int fd)
 			close(x);
 	}
 	/* Execute mpg123, but buffer if it's a net connection */
-	if (strncmp(filename, "http://", 7)) 
-	    execl(MPG_123, MPG_123, "-q", "-s", "-b", "1024", "--mono", "-r", "8000", filename, NULL);
-	else
-	    execl(MPG_123, MPG_123, "-q", "-s", "--mono", "-r", "8000", filename, NULL);
+	if (!strncasecmp(filename, "http://", 7)) {
+		/* Most commonly installed in /usr/local/bin */
+	    execl(LOCAL_MPG_123, "mpg123", "-q", "-s", "-b", "1024", "-f", "8192", "--mono", "-r", "8000", filename, (char *)NULL);
+		/* But many places has it in /usr/bin */
+	    execl(MPG_123, "mpg123", "-q", "-s", "-b", "1024","-f", "8192", "--mono", "-r", "8000", filename, (char *)NULL);
+		/* As a last-ditch effort, try to use PATH */
+	    execlp("mpg123", "mpg123", "-q", "-s", "-b", "1024",  "-f", "8192", "--mono", "-r", "8000", filename, (char *)NULL);
+	}
+	else {
+		/* Most commonly installed in /usr/local/bin */
+	    execl(MPG_123, "mpg123", "-q", "-s", "-f", "8192", "--mono", "-r", "8000", filename, (char *)NULL);
+		/* But many places has it in /usr/bin */
+	    execl(LOCAL_MPG_123, "mpg123", "-q", "-s", "-f", "8192", "--mono", "-r", "8000", filename, (char *)NULL);
+		/* As a last-ditch effort, try to use PATH */
+	    execlp("mpg123", "mpg123", "-q", "-s", "-f", "8192", "--mono", "-r", "8000", filename, (char *)NULL);
+	}
 	ast_log(LOG_WARNING, "Execute of mpg123 failed\n");
 	return -1;
 }
 
+static int timed_read(int fd, void *data, int datalen, int timeout)
+{
+	int res;
+	struct pollfd fds[1];
+	fds[0].fd = fd;
+	fds[0].events = POLLIN;
+	res = poll(fds, 1, timeout);
+	if (res < 1) {
+		ast_log(LOG_NOTICE, "Poll timed out/errored out with %d\n", res);
+		return -1;
+	}
+	return read(fd, data, datalen);
+	
+}
+
 static int mp3_exec(struct ast_channel *chan, void *data)
 {
 	int res=0;
 	struct localuser *u;
 	int fds[2];
-	int rfds[1 + AST_MAX_FDS];
 	int ms = -1;
 	int pid = -1;
-	int us;
-	int exception;
 	int owriteformat;
-	struct timeval tv;
-	struct timeval last;
+	int timeout = 2000;
+	struct timeval now, next;
 	struct ast_frame *f;
-	int x;
 	struct myframe {
 		struct ast_frame f;
 		char offset[AST_FRIENDLY_OFFSET];
-		char frdata[160];
+		short frdata[160];
 	} myf;
-	last.tv_usec = 0;
-	last.tv_sec = 0;
 	if (!data) {
 		ast_log(LOG_WARNING, "MP3 Playback requires an argument (filename)\n");
 		return -1;
@@ -107,66 +129,46 @@ static int mp3_exec(struct ast_channel *chan, void *data)
 		return -1;
 	}
 	
+	gettimeofday(&now, NULL);
 	res = mp3play((char *)data, fds[1]);
+	if (!strncasecmp((char *)data, "http://", 7)) {
+		timeout = 10000;
+	}
+	/* Wait 1000 ms first */
+	next = now;
+	next.tv_sec += 1;
 	if (res >= 0) {
 		pid = res;
 		/* Order is important -- there's almost always going to be mp3...  we want to prioritize the
 		   user */
-		rfds[AST_MAX_FDS] = fds[0];
 		for (;;) {
-			CHECK_BLOCKING(chan);
-			for (x=0;x<AST_MAX_FDS;x++) 
-				rfds[x] = chan->fds[x];
-			res = ast_waitfor_n_fd(rfds, AST_MAX_FDS+1, &ms, &exception);
-			chan->blocking = 0;
-			if (res < 1) {
-				ast_log(LOG_DEBUG, "Hangup detected\n");
-				res = -1;
-				break;
-			}
-			for(x=0;x<AST_MAX_FDS;x++) 
-				if (res == chan->fds[x])
-					break;
-
-			if (x < AST_MAX_FDS) {
-				if (exception)
-					chan->exception = 1;
-				f = ast_read(chan);
-				if (!f) {
-					ast_log(LOG_DEBUG, "Null frame == hangup() detected\n");
-					res = -1;
-					break;
+			gettimeofday(&now, NULL);
+			ms = (next.tv_sec - now.tv_sec) * 1000;
+			ms += (next.tv_usec - now.tv_usec) / 1000;
+#if 0
+			printf("ms: %d\n", ms);
+#endif			
+			if (ms <= 0) {
+#if 0
+				{
+					static struct timeval last;
+					struct timeval tv;
+					gettimeofday(&tv, NULL);
+					printf("Since last: %ld\n", (tv.tv_sec - last.tv_sec) * 1000 + (tv.tv_usec - last.tv_usec) / 1000);
+					last = tv;
 				}
-				if (f->frametype == AST_FRAME_DTMF) {
-					ast_log(LOG_DEBUG, "User pressed a key\n");
-					ast_frfree(f);
-					res = 0;
-					break;
-				}
-				ast_frfree(f);
-			} else if (res == fds[0]) {
-				gettimeofday(&tv, NULL);
-				if (last.tv_sec || last.tv_usec) {
-					/* We should wait at least a frame length */
-					us = sizeof(myf.frdata) / 16 * 1000;
-					/* Subtract 1,000,000 us for each second late we've passed */
-					us -= (tv.tv_sec - last.tv_sec) * 1000000;
-					/* And one for each us late we've passed */
-					us -= (tv.tv_usec - last.tv_usec);
-					/* Sleep that long if needed */
-					if (us > 0)
-						usleep(us);
-				}
-				last = tv;
-				res = read(fds[0], myf.frdata, sizeof(myf.frdata));
+#endif
+				res = timed_read(fds[0], myf.frdata, sizeof(myf.frdata), timeout);
 				if (res > 0) {
 					myf.f.frametype = AST_FRAME_VOICE;
 					myf.f.subclass = AST_FORMAT_SLINEAR;
 					myf.f.datalen = res;
-					myf.f.timelen = res / 16;
+					myf.f.samples = res / 2;
 					myf.f.mallocd = 0;
 					myf.f.offset = AST_FRIENDLY_OFFSET;
 					myf.f.src = __PRETTY_FUNCTION__;
+					myf.f.delivery.tv_sec = 0;
+					myf.f.delivery.tv_usec = 0;
 					myf.f.data = myf.frdata;
 					if (ast_write(chan, &myf.f) < 0) {
 						res = -1;
@@ -175,11 +177,38 @@ static int mp3_exec(struct ast_channel *chan, void *data)
 				} else {
 					ast_log(LOG_DEBUG, "No more mp3\n");
 					res = 0;
+					break;
 				}
+				next.tv_usec += res / 2 * 125;
+				if (next.tv_usec >= 1000000) {
+					next.tv_usec -= 1000000;
+					next.tv_sec++;
+				}
+#if 0
+				printf("Next: %d\n", ms);
+#endif				
 			} else {
-				ast_log(LOG_DEBUG, "HuhHHH?\n");
-				res = -1;
-				break;
+				ms = ast_waitfor(chan, ms);
+				if (ms < 0) {
+					ast_log(LOG_DEBUG, "Hangup detected\n");
+					res = -1;
+					break;
+				}
+				if (ms) {
+					f = ast_read(chan);
+					if (!f) {
+						ast_log(LOG_DEBUG, "Null frame == hangup() detected\n");
+						res = -1;
+						break;
+					}
+					if (f->frametype == AST_FRAME_DTMF) {
+						ast_log(LOG_DEBUG, "User pressed a key\n");
+						ast_frfree(f);
+						res = 0;
+						break;
+					}
+					ast_frfree(f);
+				} 
 			}
 		}
 	}
