@@ -11,6 +11,7 @@
  * the GNU General Public License
  */
  
+#include <asterisk/lock.h>
 #include <asterisk/file.h>
 #include <asterisk/logger.h>
 #include <asterisk/channel.h>
@@ -20,7 +21,6 @@
 #include <asterisk/image.h>
 #include <string.h>
 #include <stdlib.h>
-#include <pthread.h>
 
 static char *tdesc = "Send URL Applications";
 
@@ -51,13 +51,15 @@ static int sendurl_exec(struct ast_channel *chan, void *data)
 	char *options;
 	int option_wait=0;
 	struct ast_frame *f;
+	char *stringp=NULL;
 	if (!data || !strlen((char *)data)) {
 		ast_log(LOG_WARNING, "SendURL requires an argument (URL)\n");
 		return -1;
 	}
 	strncpy(tmp, (char *)data, sizeof(tmp)-1);
-	strtok(tmp, "|");
-	options = strtok(NULL, "|");
+	stringp=tmp;
+	strsep(&stringp, "|");
+	options = strsep(&stringp, "|");
 	if (options && !strcasecmp(options, "wait"))
 		option_wait = 1;
 	LOCAL_USER_ADD(u);
