@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
  *
- * Version Info: $Id: chan_h323.h,v 1.27 2004/09/21 18:42:37 jeremy Exp $
+ * Version Info: $Id: chan_h323.h,v 1.27.2.2 2004/11/04 23:14:26 russell Exp $
  */
 
 #include <arpa/inet.h>
@@ -114,7 +114,7 @@ typedef struct rtp_info {
 
 /* This is a callback prototype function, called pass
    DTMF down the RTP. */
-typedef int (*send_digit_cb)(unsigned, char);
+typedef int (*send_digit_cb)(unsigned, char, const char *);
 extern send_digit_cb	on_send_digit; 
 
 /* This is a callback prototype function, called to collect
