@@ -4,34 +4,48 @@
  * details.  THERE IS ABSOLUTELY NO WARRANTY FOR THIS SOFTWARE.
  */
 
-/*$Header: /usr/share/cvsroot/asterisk-ng/codecs/gsm/inc/config.h,v 1.1 1999/11/15 06:09:04 markster Exp $*/
+/*$Header: /usr/cvsroot/asterisk/codecs/gsm/inc/config.h,v 1.19 2004/06/26 03:50:14 markster Exp $*/
 
 #ifndef	CONFIG_H
 #define	CONFIG_H
 
-/*efine	SIGHANDLER_T	int 		/* signal handlers are void	*/
-/*efine HAS_SYSV_SIGNAL	1		/* sigs not blocked/reset?	*/
+#if 0
+efine	SIGHANDLER_T	int 		/* signal handlers are void	*/
+efine HAS_SYSV_SIGNAL	1		/* sigs not blocked/reset?	*/
+#endif
 
 #define	HAS_STDLIB_H	1		/* /usr/include/stdlib.h	*/
-/*efine	HAS_LIMITS_H	1		/* /usr/include/limits.h	*/
+#if 0
+efine	HAS_LIMITS_H	1		/* /usr/include/limits.h	*/
+#endif
 #define	HAS_FCNTL_H	1		/* /usr/include/fcntl.h		*/
-/*efine	HAS_ERRNO_DECL	1		/* errno.h declares errno	*/
+#if 0
+efine	HAS_ERRNO_DECL	1		/* errno.h declares errno	*/
+#endif
 
 #define	HAS_FSTAT 	1		/* fstat syscall		*/
 #define	HAS_FCHMOD 	1		/* fchmod syscall		*/
 #define	HAS_CHMOD 	1		/* chmod syscall		*/
 #define	HAS_FCHOWN 	1		/* fchown syscall		*/
 #define	HAS_CHOWN 	1		/* chown syscall		*/
-/*efine	HAS__FSETMODE 	1		/* _fsetmode -- set file mode	*/
+#if 0
+efine	HAS__FSETMODE 	1		/* _fsetmode -- set file mode	*/
+#endif
 
 #define	HAS_STRING_H 	1		/* /usr/include/string.h 	*/
-/*efine	HAS_STRINGS_H	1		/* /usr/include/strings.h 	*/
+#if 0
+efine	HAS_STRINGS_H	1		/* /usr/include/strings.h 	*/
+#endif
 
 #define	HAS_UNISTD_H	1		/* /usr/include/unistd.h	*/
 #define	HAS_UTIME	1		/* POSIX utime(path, times)	*/
-/*efine	HAS_UTIMES	1		/* use utimes()	syscall instead	*/
+#if 0
+efine	HAS_UTIMES	1		/* use utimes()	syscall instead	*/
+#endif
 #define	HAS_UTIME_H	1		/* UTIME header file		*/
-/*efine	HAS_UTIMBUF	1		/* struct utimbuf		*/
-/*efine	HAS_UTIMEUSEC   1		/* microseconds in utimbuf?	*/
+#if 0
+efine	HAS_UTIMBUF	1		/* struct utimbuf		*/
+efine	HAS_UTIMEUSEC   1		/* microseconds in utimbuf?	*/
+#endif
 
 #endif	/* CONFIG_H */
