@@ -11,11 +11,13 @@
  * the GNU General Public License
  */
  
+#include <asterisk/lock.h>
 #include <asterisk/channel.h>
 #include <asterisk/file.h>
 #include <asterisk/logger.h>
 #include <asterisk/sched.h>
 #include <asterisk/module.h>
+#include <netinet/in.h>
 #include <arpa/inet.h>
 #include <stdlib.h>
 #include <sys/time.h>
@@ -23,35 +25,34 @@
 #include <unistd.h>
 #include <errno.h>
 #include <string.h>
-#include <pthread.h>
+#ifdef __linux__
 #include <endian.h>
+#else
+#include <machine/endian.h>
+#endif
 
-/* Some Ideas for this code came from makewave.c by Jeffery Chilton */
+/* Some Ideas for this code came from makewave.c by Jeffrey Chilton */
 
 /* Portions of the conversion code are by guido@sienanet.it */
 
 struct ast_filestream {
 	void *reserved[AST_RESERVED_POINTERS];
-	/* Believe it or not, we must decode/recode to account for the
-	   weird MS format */
 	/* This is what a filestream means to us */
 	int fd; /* Descriptor */
 	int bytes;
-	struct ast_channel *owner;
+	int needsgain;
 	struct ast_frame fr;				/* Frame information */
 	char waste[AST_FRIENDLY_OFFSET];	/* Buffer for sending frames, etc */
 	char empty;							/* Empty character */
-	short buf[160];				/* Two Real GSM Frames */
+	short buf[160];	
 	int foffset;
 	int lasttimeout;
+	int maxlen;
 	struct timeval last;
-	int adj;
-	struct ast_filestream *next;
 };
 
 
-static struct ast_filestream *glist = NULL;
-static pthread_mutex_t wav_lock = PTHREAD_MUTEX_INITIALIZER;
+AST_MUTEX_DEFINE_STATIC(wav_lock);
 static int glistcnt = 0;
 
 static char *name = "wav";
@@ -60,6 +61,8 @@ static char *exts = "wav";
 
 #define BLOCKSIZE 160
 
+#define GAIN 2		/* 2^GAIN is the multiple to increase the volume by */
+
 #if __BYTE_ORDER == __LITTLE_ENDIAN
 #define htoll(b) (b)
 #define htols(b) (b)
@@ -124,7 +127,7 @@ static int check_header(int fd)
 		ast_log(LOG_WARNING, "Read failed (formtype)\n");
 		return -1;
 	}
-	if (ltohl(hsize) != 16) {
+	if (ltohl(hsize) < 16) {
 		ast_log(LOG_WARNING, "Unexpected header size %d\n", ltohl(hsize));
 		return -1;
 	}
@@ -167,34 +170,63 @@ static int check_header(int fd)
 		return -1;
 	}
 	if (read(fd, &bisam, 2) != 2) {
-		ast_log(LOG_WARNING, "Read failed (Bits Per Sample): &d\n", ltohs(bisam));
+		ast_log(LOG_WARNING, "Read failed (Bits Per Sample): %d\n", ltohs(bisam));
 		return -1;
 	}
-        /* Begin data chunk */
-	if (read(fd, &data, 4) != 4) {
+	// Skip any additional header
+	if ( lseek(fd,ltohl(hsize)-16,SEEK_CUR) == -1 ) {
+		ast_log(LOG_WARNING, "Failed to skip remaining header bytes: %d\n", ltohl(hsize)-16 );
+		return -1;
+	}
+	// Skip any facts and get the first data block
+	for(;;)
+	{ 
+            char buf[4];
+	    
+	    /* Begin data chunk */
+	    if (read(fd, &buf, 4) != 4) {
 		ast_log(LOG_WARNING, "Read failed (data)\n");
 		return -1;
-	}
-	if (memcmp(&data, "data", 4)) {
-		ast_log(LOG_WARNING, "Does not say data\n");
-		return -1;
-	}
-	/* Ignore the data length */
-	if (read(fd, &data, 4) != 4) {
+	    }
+	    /* Data has the actual length of data in it */
+	    if (read(fd, &data, 4) != 4) {
 		ast_log(LOG_WARNING, "Read failed (data)\n");
 		return -1;
+	    }
+	    data = ltohl(data);
+	    if( memcmp(buf, "data", 4) == 0 ) break;
+	    if( memcmp(buf, "fact", 4) != 0 ) {
+		ast_log(LOG_WARNING, "Unknown block - not fact or data\n");
+		return -1;
+	    }
+	    if ( lseek(fd,data,SEEK_CUR) == -1 ) {
+		ast_log(LOG_WARNING, "Failed to skip fact block: %d\n", data );
+		return -1;
+	    }
 	}
-	return 0;
+#if 0
+	curpos = lseek(fd, 0, SEEK_CUR);
+	truelength = lseek(fd, 0, SEEK_END);
+	lseek(fd, curpos, SEEK_SET);
+	truelength -= curpos;
+#endif	
+	return data;
 }
 
-static int update_header(int fd, int bytes)
+static int update_header(int fd)
 {
-	int cur;
-	int datalen = htoll(bytes);
-	/* int filelen = htoll(52 + ((bytes + 1) & ~0x1)); */
-	int filelen = htoll(36 + bytes);
+	off_t cur,end;
+	int datalen,filelen,bytes;
+	
 	
 	cur = lseek(fd, 0, SEEK_CUR);
+	end = lseek(fd, 0, SEEK_END);
+	/* data starts 44 bytes in */
+	bytes = end - 44;
+	datalen = htoll(bytes);
+	/* chunk size is bytes of data plus 36 bytes of header */
+	filelen = htoll(36 + bytes);
+	
 	if (cur < 0) {
 		ast_log(LOG_WARNING, "Unable to find our position\n");
 		return -1;
@@ -233,6 +265,7 @@ static int write_header(int fd)
 	unsigned short bisam = htols(16);
 	unsigned int size = htoll(0);
 	/* Write a wav header, ignoring sizes which will be filled in later */
+	lseek(fd,0,SEEK_SET);
 	if (write(fd, "RIFF", 4) != 4) {
 		ast_log(LOG_WARNING, "Unable to write header\n");
 		return -1;
@@ -292,29 +325,26 @@ static struct ast_filestream *wav_open(int fd)
 	struct ast_filestream *tmp;
 	if ((tmp = malloc(sizeof(struct ast_filestream)))) {
 		memset(tmp, 0, sizeof(struct ast_filestream));
-		if (check_header(fd)) {
+		if ((tmp->maxlen = check_header(fd)) < 0) {
 			free(tmp);
 			return NULL;
 		}
-		if (ast_pthread_mutex_lock(&wav_lock)) {
+		if (ast_mutex_lock(&wav_lock)) {
 			ast_log(LOG_WARNING, "Unable to lock wav list\n");
 			free(tmp);
 			return NULL;
 		}
-		tmp->next = glist;
-		glist = tmp;
 		tmp->fd = fd;
-		tmp->owner = NULL;
+		tmp->needsgain = 1;
 		tmp->fr.data = tmp->buf;
 		tmp->fr.frametype = AST_FRAME_VOICE;
 		tmp->fr.subclass = AST_FORMAT_SLINEAR;
 		/* datalen will vary for each frame */
 		tmp->fr.src = name;
 		tmp->fr.mallocd = 0;
-		tmp->lasttimeout = -1;
 		tmp->bytes = 0;
 		glistcnt++;
-		ast_pthread_mutex_unlock(&wav_lock);
+		ast_mutex_unlock(&wav_lock);
 		ast_update_use_count();
 	}
 	return tmp;
@@ -332,142 +362,136 @@ static struct ast_filestream *wav_rewrite(int fd, char *comment)
 			free(tmp);
 			return NULL;
 		}
-		if (ast_pthread_mutex_lock(&wav_lock)) {
+		if (ast_mutex_lock(&wav_lock)) {
 			ast_log(LOG_WARNING, "Unable to lock wav list\n");
 			free(tmp);
 			return NULL;
 		}
-		tmp->next = glist;
-		glist = tmp;
 		tmp->fd = fd;
-		tmp->owner = NULL;
-		tmp->lasttimeout = -1;
 		glistcnt++;
-		ast_pthread_mutex_unlock(&wav_lock);
+		ast_mutex_unlock(&wav_lock);
 		ast_update_use_count();
 	} else
 		ast_log(LOG_WARNING, "Out of memory\n");
 	return tmp;
 }
 
-static struct ast_frame *wav_read(struct ast_filestream *s)
-{
-	return NULL;
-}
-
 static void wav_close(struct ast_filestream *s)
 {
-	struct ast_filestream *tmp, *tmpl = NULL;
 	char zero = 0;
-	if (ast_pthread_mutex_lock(&wav_lock)) {
+	if (ast_mutex_lock(&wav_lock)) {
 		ast_log(LOG_WARNING, "Unable to lock wav list\n");
 		return;
 	}
-	tmp = glist;
-	while(tmp) {
-		if (tmp == s) {
-			if (tmpl)
-				tmpl->next = tmp->next;
-			else
-				glist = tmp->next;
-			break;
-		}
-		tmpl = tmp;
-		tmp = tmp->next;
-	}
 	glistcnt--;
-	if (s->owner) {
-		s->owner->stream = NULL;
-		if (s->owner->streamid > -1)
-			ast_sched_del(s->owner->sched, s->owner->streamid);
-		s->owner->streamid = -1;
-	}
-	ast_pthread_mutex_unlock(&wav_lock);
+	ast_mutex_unlock(&wav_lock);
 	ast_update_use_count();
-	if (!tmp) 
-		ast_log(LOG_WARNING, "Freeing a filestream we don't seem to own\n");
 	/* Pad to even length */
 	if (s->bytes & 0x1)
 		write(s->fd, &zero, 1);
 	close(s->fd);
 	free(s);
-#if 0
-	printf("bytes = %d\n", s->bytes);
-#endif
+	s = NULL;
 }
 
-static int ast_read_callback(void *data)
+static struct ast_frame *wav_read(struct ast_filestream *s, int *whennext)
 {
-	int retval = 0;
 	int res;
 	int delay;
-	struct ast_filestream *s = data;
+	int x;
+	short tmp[sizeof(s->buf) / 2];
+	int bytes = sizeof(tmp);
+	off_t here;
 	/* Send a frame from the file to the appropriate channel */
+	here = lseek(s->fd, 0, SEEK_CUR);
+	if ((s->maxlen - here) < bytes)
+		bytes = s->maxlen - here;
+	if (bytes < 0)
+		bytes = 0;
+/* 	ast_log(LOG_DEBUG, "here: %d, maxlen: %d, bytes: %d\n", here, s->maxlen, bytes); */
 	
-	if ( (res = read(s->fd, s->buf, sizeof(s->buf))) <= 0 ) {
+	if ( (res = read(s->fd, tmp, bytes)) <= 0 ) {
 		if (res) {
 			ast_log(LOG_WARNING, "Short read (%d) (%s)!\n", res, strerror(errno));
 		}
-		s->owner->streamid = -1;
-		return 0;
+		return NULL;
 	}
-		
-	delay = res / 16;
+
+#if __BYTE_ORDER == __BIG_ENDIAN
+	for( x = 0; x < sizeof(tmp)/2; x++) tmp[x] = (tmp[x] << 8) | ((tmp[x] & 0xff00) >> 8);
+#endif
+
+	if (s->needsgain) {
+		for (x=0;x<sizeof(tmp)/2;x++)
+			if (tmp[x] & ((1 << GAIN) - 1)) {
+				/* If it has data down low, then it's not something we've artificially increased gain
+				   on, so we don't need to gain adjust it */
+				s->needsgain = 0;
+			}
+	}
+	if (s->needsgain) {
+		for (x=0;x<sizeof(tmp)/2;x++) {
+			s->buf[x] = tmp[x] >> GAIN;
+		}
+	} else {
+		memcpy(s->buf, tmp, sizeof(s->buf));
+	}
+			
+	delay = res / 2;
 	s->fr.frametype = AST_FRAME_VOICE;
 	s->fr.subclass = AST_FORMAT_SLINEAR;
 	s->fr.offset = AST_FRIENDLY_OFFSET;
 	s->fr.datalen = res;
 	s->fr.data = s->buf;
 	s->fr.mallocd = 0;
-	s->fr.timelen = delay;
-
-	/* Lastly, process the frame */	
-	if (delay != s->lasttimeout) {
-		s->owner->streamid = ast_sched_add(s->owner->sched, delay, ast_read_callback, s);
-		s->lasttimeout = delay;
-	} else {
-		retval = -1;
-	}
-	
-	
-	if (ast_write(s->owner, &s->fr)) {
-		ast_log(LOG_WARNING, "Failed to write frame\n");
-		s->owner->streamid = -1;
-		return 0;
-	}
-	
-	return retval;
-}
-
-static int wav_apply(struct ast_channel *c, struct ast_filestream *s)
-{
-	/* Select our owner for this stream, and get the ball rolling. */
-	s->owner = c;
-	ast_read_callback(s);
-	return 0;
+	s->fr.samples = delay;
+	*whennext = delay;
+	return &s->fr;
 }
 
 static int wav_write(struct ast_filestream *fs, struct ast_frame *f)
 {
 	int res = 0;
-#if 0
-	int size = 0;
-#endif
+	int x;
+	short tmp[8000], *tmpi;
+	float tmpf;
 	if (f->frametype != AST_FRAME_VOICE) {
 		ast_log(LOG_WARNING, "Asked to write non-voice frame!\n");
 		return -1;
 	}
 	if (f->subclass != AST_FORMAT_SLINEAR) {
-		ast_log(LOG_WARNING, "Asked to write non-GSM frame (%d)!\n", f->subclass);
+		ast_log(LOG_WARNING, "Asked to write non-SLINEAR frame (%d)!\n", f->subclass);
 		return -1;
 	}
+	if (f->datalen > sizeof(tmp)) {
+		ast_log(LOG_WARNING, "Data length is too long\n");
+		return -1;
+	}
+	if (!f->datalen)
+		return -1;
 
 #if 0
 	printf("Data Length: %d\n", f->datalen);
 #endif	
 
 	if (fs->buf) {
-		if ((write (fs->fd, f->data, f->datalen) != f->datalen) ) {
+		tmpi = f->data;
+		/* Volume adjust here to accomodate */
+		for (x=0;x<f->datalen/2;x++) {
+			tmpf = ((float)tmpi[x]) * ((float)(1 << GAIN));
+			if (tmpf > 32767.0)
+				tmpf = 32767.0;
+			if (tmpf < -32768.0)
+				tmpf = -32768.0;
+			tmp[x] = tmpf;
+			tmp[x] &= ~((1 << GAIN) - 1);
+
+#if __BYTE_ORDER == __BIG_ENDIAN
+			tmp[x] = (tmp[x] << 8) | ((tmp[x] & 0xff00) >> 8);
+#endif
+
+		}
+		if ((write (fs->fd, tmp, f->datalen) != f->datalen) ) {
 			ast_log(LOG_WARNING, "Bad write (%d): %s\n", res, strerror(errno));
 			return -1;
 		}
@@ -477,12 +501,50 @@ static int wav_write(struct ast_filestream *fs, struct ast_frame *f)
 	}
 	
 	fs->bytes += f->datalen;
-	update_header(fs->fd, fs->bytes);
+	update_header(fs->fd);
 		
 	return 0;
 
 }
 
+static int wav_seek(struct ast_filestream *fs, long sample_offset, int whence)
+{
+	off_t min,max,cur;
+	long offset=0,samples;
+	
+	samples = sample_offset * 2; /* SLINEAR is 16 bits mono, so sample_offset * 2 = bytes */
+	min = 44; /* wav header is 44 bytes */
+	cur = lseek(fs->fd, 0, SEEK_CUR);
+	max = lseek(fs->fd, 0, SEEK_END);
+	if (whence == SEEK_SET)
+		offset = samples + min;
+	else if (whence == SEEK_CUR || whence == SEEK_FORCECUR)
+		offset = samples + cur;
+	else if (whence == SEEK_END)
+		offset = max - samples;
+        if (whence != SEEK_FORCECUR) {
+		offset = (offset > max)?max:offset;
+	}
+	// always protect the header space.
+	offset = (offset < min)?min:offset;
+	return lseek(fs->fd,offset,SEEK_SET);
+}
+
+static int wav_trunc(struct ast_filestream *fs)
+{
+	if(ftruncate(fs->fd, lseek(fs->fd,0,SEEK_CUR)))
+		return -1;
+	return update_header(fs->fd);
+}
+
+static long wav_tell(struct ast_filestream *fs)
+{
+	off_t offset;
+	offset = lseek(fs->fd, 0, SEEK_CUR);
+	/* subtract header size to get samples, then divide by 2 for 16 bit samples */
+	return (offset - 44)/2;
+}
+
 static char *wav_getcomment(struct ast_filestream *s)
 {
 	return NULL;
@@ -493,8 +555,10 @@ int load_module()
 	return ast_format_register(name, exts, AST_FORMAT_SLINEAR,
 								wav_open,
 								wav_rewrite,
-								wav_apply,
 								wav_write,
+								wav_seek,
+								wav_trunc,
+								wav_tell,
 								wav_read,
 								wav_close,
 								wav_getcomment);
@@ -504,32 +568,18 @@ int load_module()
 
 int unload_module()
 {
-	struct ast_filestream *tmp, *tmpl;
-	if (ast_pthread_mutex_lock(&wav_lock)) {
-		ast_log(LOG_WARNING, "Unable to lock wav list\n");
-		return -1;
-	}
-	tmp = glist;
-	while(tmp) {
-		if (tmp->owner)
-			ast_softhangup(tmp->owner);
-		tmpl = tmp;
-		tmp = tmp->next;
-		free(tmpl);
-	}
-	ast_pthread_mutex_unlock(&wav_lock);
 	return ast_format_unregister(name);
 }	
 
 int usecount()
 {
 	int res;
-	if (ast_pthread_mutex_lock(&wav_lock)) {
+	if (ast_mutex_lock(&wav_lock)) {
 		ast_log(LOG_WARNING, "Unable to lock wav list\n");
 		return -1;
 	}
 	res = glistcnt;
-	ast_pthread_mutex_unlock(&wav_lock);
+	ast_mutex_unlock(&wav_lock);
 	return res;
 }
 
