@@ -20,14 +20,21 @@
 #include <asterisk/channel.h>
 #include <sys/time.h>
 
+#define AST_CDR_FLAG_POSTED			(1 << 1)
+#define AST_CDR_FLAG_LOCKED			(1 << 2)
+#define AST_CDR_FLAG_CHILD			(1 << 3)
+
 #define AST_CDR_NOANSWER			(1 << 0)
 #define AST_CDR_BUSY				(1 << 1)
 #define AST_CDR_ANSWERED			(1 << 2)
+#define AST_CDR_FAILED				(1 << 3)
 
 //! AMA Flags
 #define AST_CDR_OMIT				(1)
 #define AST_CDR_BILLING				(2)
-#define AST_CDR_DOCUMENTATION		(3)
+#define AST_CDR_DOCUMENTATION			(3)
+
+#define AST_MAX_USER_FIELD			256
 
 struct ast_channel;
 
@@ -65,8 +72,13 @@ struct ast_cdr {
 	int amaflags;				
 	/*! What account number to use */
 	char accountcode[20];			
-	/*! Whether or not the record has been posted */
-	int posted;				
+	/*! flags */
+	int flags;				
+	/* Unique Channel Identifier */
+	char uniqueid[32];
+	/* User field */
+	char userfield[AST_MAX_USER_FIELD];
+	struct ast_cdr *next;
 };
 
 typedef int (*ast_cdrbe)(struct ast_cdr *cdr);
@@ -92,6 +104,15 @@ extern void ast_cdr_free(struct ast_cdr *cdr);
  */
 extern int ast_cdr_init(struct ast_cdr *cdr, struct ast_channel *chan);
 
+//! Initialize based on a channel
+/*! 
+ * \param cdr Call Detail Record to use for channel
+ * \param chan Channel to bind CDR with
+ * Initializes a CDR and associates it with a particular channel
+ * Return is negligible.  (returns 0 by default)
+ */
+extern int ast_cdr_setcid(struct ast_cdr *cdr, struct ast_channel *chan);
+
 //! Register a CDR handling engine
 /*!
  * \param name name associated with the particular CDR handler
@@ -131,6 +152,21 @@ extern void ast_cdr_answer(struct ast_cdr *cdr);
  */
 extern void ast_cdr_busy(struct ast_cdr *cdr);
 
+//! Fail a call
+/*!
+ * \param cdr the cdr you wish to associate with the call
+ * Returns nothing important
+ */
+extern void ast_cdr_failed(struct ast_cdr *cdr);
+
+//! Save the result of the call based on the AST_CAUSE_*
+/*!
+ * \param cdr the cdr you wish to associate with the call
+ * Returns nothing important
+ * \param cause the AST_CAUSE_*
+ */
+extern int ast_cdr_disposition(struct ast_cdr *cdr, int cause);
+	
 //! End a call
 /*!
  * \param cdr the cdr you have associated the call with
@@ -181,6 +217,14 @@ extern int ast_cdr_amaflags2int(char *flag);
  */
 extern char *ast_cdr_disp2str(int disposition);
 
+//! Reset the detail record, optionally posting it first
+/*!
+ * \param cdr which cdr to act upon
+ * \param flags |AST_CDR_FLAG_POSTED whether or not to post the cdr first before resetting it
+ *              |AST_CDR_FLAG_LOCKED whether or not to reset locked CDR's
+ */
+extern void ast_cdr_reset(struct ast_cdr *cdr, int flags);
+
 //! Flags to a string
 /*!
  * \param flags binary flag
@@ -189,8 +233,27 @@ extern char *ast_cdr_disp2str(int disposition);
  */
 extern char *ast_cdr_flags2str(int flags);
 
+extern int ast_cdr_setaccount(struct ast_channel *chan, char *account);
+extern int ast_cdr_setamaflags(struct ast_channel *chan, char *account);
+
+
+extern int ast_cdr_setuserfield(struct ast_channel *chan, char *userfield);
+extern int ast_cdr_appenduserfield(struct ast_channel *chan, char *userfield);
+
+
+/* Update CDR on a channel */
+extern int ast_cdr_update(struct ast_channel *chan);
+
+
 extern int ast_default_amaflags;
 
 extern char ast_default_accountcode[20];
 
+#define ast_cdr_compare_flag(flags, flag) (flags & (flag))
+#define ast_cdr_has_flag(cdr, flag) ((cdr)->flags & (flag))
+#define ast_cdr_add_flag(cdr, flag) ((cdr)->flags |= (flag))
+#define ast_cdr_del_flag(cdr, flag) ((cdr)->flags &= ~(flag))
+
+extern struct ast_cdr *ast_cdr_append(struct ast_cdr *cdr, struct ast_cdr *newcdr);
+
 #endif /* _CDR_H */
