@@ -3,9 +3,9 @@
  *
  * General Asterisk channel definitions.
  * 
- * Copyright (C) 1999, Mark Spencer
+ * Copyright (C) 1999-2004, Digium, Inc.
  *
- * Mark Spencer <markster@linux-support.net>
+ * Mark Spencer <markster@digium.com>
  *
  * This program is free software, distributed under the terms of
  * the GNU General Public License
@@ -16,73 +16,26 @@
 
 #include <asterisk/frame.h>
 #include <asterisk/sched.h>
+#include <asterisk/chanvars.h>
+#include <unistd.h>
 #include <setjmp.h>
-#include <pthread.h>
+#if defined(__APPLE__)
+#include <asterisk/poll-compat.h>
+#else
+#include <sys/poll.h>
+#endif
 
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
 #endif
 
-#include <pthread.h>
-
-#ifdef DEBUG_THREADS
-
-#define TRIES 50
-
-#include <errno.h>
-#include <string.h>
-#include <stdio.h>
-#include <unistd.h>
-
-struct mutex_info {
-	pthread_mutex_t *mutex;
-	char *file;
-	int lineno;
-	char *func;
-	struct mutex_info *next;
-};
-
-static inline int __ast_pthread_mutex_lock(char *filename, int lineno, char *func, pthread_mutex_t *t) {
-	int res;
-	int tries = TRIES;
-	do {
-		res = pthread_mutex_trylock(t);
-		/* If we can't run, yield */
-		if (res) {
-			sched_yield();
-			usleep(1);
-		}
-	} while(res && tries--);
-	if (res) {
-		fprintf(stderr, "%s line %d (%s): Error obtaining mutex: %s\n", 
-				filename, lineno, func, strerror(res));
-		res = pthread_mutex_lock(t);
-		fprintf(stderr, "%s line %d (%s): Got it eventually...\n",
-				filename, lineno, func);
-	}
-	return res;
-}
-
-#define ast_pthread_mutex_lock(a) __ast_pthread_mutex_lock(__FILE__, __LINE__, __PRETTY_FUNCTION__, a)
-
-static inline int __ast_pthread_mutex_unlock(char *filename, int lineno, char *func, pthread_mutex_t *t) {
-	int res;
-	res = pthread_mutex_unlock(t);
-	if (res) 
-		fprintf(stderr, "%s line %d (%s): Error releasing mutex: %s\n", 
-				filename, lineno, func, strerror(res));
-	return res;
-}
-#define ast_pthread_mutex_unlock(a) __ast_pthread_mutex_unlock(__FILE__, __LINE__, __PRETTY_FUNCTION__, a)
-#else
-#define ast_pthread_mutex_lock pthread_mutex_lock
-#define ast_pthread_mutex_unlock pthread_mutex_unlock
-#endif
+#include <asterisk/lock.h>
 
 //! Max length of an extension
 #define AST_MAX_EXTENSION 80
 
 #include <asterisk/cdr.h>
+#include <asterisk/monitor.h>
 
 
 #define AST_CHANNEL_NAME 80
@@ -91,7 +44,13 @@ static inline int __ast_pthread_mutex_unlock(char *filename, int lineno, char *f
 #define MAX_LANGUAGE 20
 
 
-#define AST_MAX_FDS 4
+#define AST_MAX_FDS 8
+
+struct ast_generator {
+	void *(*alloc)(struct ast_channel *chan, void *params);
+	void (*release)(struct ast_channel *chan, void *data);
+	int (*generate)(struct ast_channel *chan, void *data, int len, int samples);
+};
 
 //! Main Channel structure associated with a channel.
 /*! 
@@ -106,9 +65,25 @@ struct ast_channel {
 	char *type;				
 	/*! File descriptor for channel -- Drivers will poll on these file descriptors, so at least one must be non -1.  */
 	int fds[AST_MAX_FDS];			
-						   
+
+	/*! Default music class */
+	char musicclass[MAX_LANGUAGE];
+
+	/*! Current generator data if there is any */
+	void *generatordata;
+	/*! Current active data generator */
+	struct ast_generator *generator;
+	/*! Whether or not the generator should be interrupted by write */
+	int writeinterrupt;
+
 	/*! Who are we bridged to, if we're bridged */
-	struct ast_channel *bridge;		
+	struct ast_channel *bridge;
+	/*! Who did we call? */
+	struct ast_channel *dialed;
+	/*! Who called us? */
+	struct ast_channel *dialing;
+	/*! Reverse the dialed link (0 false, 1 true) */
+	int reversedialed;
 	/*! Channel that will masquerade as us */
 	struct ast_channel *masq;		
 	/*! Who we are masquerading as */
@@ -117,8 +92,9 @@ struct ast_channel {
 	int cdrflags;										   
 	/*! Whether or not we're blocking */
 	int blocking;				
-	/*! Whether or not we have been hung up */
-	int softhangup;				
+	/*! Whether or not we have been hung up...  Do not set this value
+	    directly, use ast_softhangup */
+	int _softhangup;				
 	/*! Non-zero if this is a zombie channel */
 	int zombie;					
 	/*! Non-zero, set to actual time when channel is to be hung up */
@@ -126,9 +102,9 @@ struct ast_channel {
 	/*! If anyone is blocking, this is them */
 	pthread_t blocker;			
 	/*! Lock, can be used to lock a channel for some operations */
-	pthread_mutex_t lock;			
+	ast_mutex_t lock;			
 	/*! Procedure causing blocking */
-	char *blockproc;			
+	const char *blockproc;			
 
 	/*! Current application */
 	char *appl;				
@@ -145,16 +121,24 @@ struct ast_channel {
 	int streamid;				
 	/*! Stream itself. */
 	struct ast_filestream *stream;		
+	/*! For streaming playback, the schedule ID */
+	int vstreamid;				
+	/*! Stream itself. */
+	struct ast_filestream *vstream;		
 	/*! Original writer format */
 	int oldwriteformat;			
+	
+	/*! Timing fd */
+	int timingfd;
+	int (*timingfunc)(void *data);
+	void *timingdata;
 
-
-	/*! State of line */
-	int state;				
+	/*! State of line -- Don't write directly, use ast_setstate */
+	int _state;				
 	/*! Number of rings so far */
 	int rings;				
 	/*! Current level of application */
-	int stack;				
+	int stack;
 
 
 	/*! Kinds of data this channel can natively handle */
@@ -168,13 +152,25 @@ struct ast_channel {
 	/*! Malloc'd Dialed Number Identifier */
 	char *dnid;				
 	/*! Malloc'd Caller ID */
-	char *callerid;				
+	char *callerid;
 	/*! Malloc'd ANI */
 	char *ani;			
+	/*! Malloc'd RDNIS */
+	char *rdnis;
+	/*! Hide callerid from user */
+	int restrictcid;
+	/*! Callerid presentation/screening */
+	int callingpres;
 
 	
 	/*! Current extension context */
 	char context[AST_MAX_EXTENSION];	
+	/*! Current non-macro context */
+	char macrocontext[AST_MAX_EXTENSION];	
+	/*! Current non-macro extension */
+	char macroexten[AST_MAX_EXTENSION];
+	/*! Current non-macro priority */
+	int macropriority;
 	/*! Current extension number */
 	char exten[AST_MAX_EXTENSION];		
 	/* Current extension priority */
@@ -205,9 +201,109 @@ struct ast_channel {
 	int	adsicpe;
 	/*! Where to forward to if asked to dial on this interface */
 	char call_forward[AST_MAX_EXTENSION];
-	/*! For easy linking */
-	struct ast_channel *next;		
 
+	/*! Tone zone */
+	struct tone_zone *zone;
+
+	/* Channel monitoring */
+	struct ast_channel_monitor *monitor;
+
+	/*! Track the read/written samples for monitor use */
+	unsigned long insmpl;
+	unsigned long outsmpl;
+
+	/* Frames in/out counters */
+	unsigned int fin;
+	unsigned int fout;
+
+	/* Unique Channel Identifier */
+	char uniqueid[32];
+
+	/* Why is the channel hanged up */
+	int hangupcause;
+	
+	/* A linked list for variables */
+	struct ast_var_t *vars;	
+	AST_LIST_HEAD(varshead,ast_var_t) varshead;
+
+	unsigned int callgroup;
+	unsigned int pickupgroup;
+
+	/*! channel flags of AST_FLAG_ type */
+	int flag;
+	
+	/*! For easy linking */
+	struct ast_channel *next;
+
+};
+
+#define AST_FLAG_DIGITAL	1	/* if the call is a digital ISDN call */
+
+static inline int ast_test_flag(struct ast_channel *chan, int mode)
+{
+	return chan->flag & mode;
+}
+
+static inline void ast_set_flag(struct ast_channel *chan, int mode)
+{
+	chan->flag |= mode;
+}
+
+static inline void ast_clear_flag(struct ast_channel *chan, int mode)
+{
+	chan->flag &= ~mode;
+}
+
+static inline void ast_set2_flag(struct ast_channel *chan, int value, int mode)
+{
+	if (value)
+		ast_set_flag(chan, mode);
+	else
+		ast_clear_flag(chan, mode);
+}
+
+static inline void ast_dup_flag(struct ast_channel *dstchan, struct ast_channel *srcchan, int mode)
+{
+	if (ast_test_flag(srcchan, mode))
+		ast_set_flag(dstchan, mode);
+	else
+		ast_clear_flag(dstchan, mode);
+}	
+
+struct ast_bridge_config {
+	int play_to_caller;
+	int play_to_callee;
+	int allowredirect_in;
+	int allowredirect_out;
+	int allowdisconnect_in;
+	int allowdisconnect_out;
+	long timelimit;
+	long play_warning;
+	long warning_freq;
+	char *warning_sound;
+	char *end_sound;
+	char *start_sound;
+	int firstpass;
+};
+
+struct chanmon;
+
+#define LOAD_OH(oh) {	\
+	oh.context = context; \
+	oh.exten = exten; \
+	oh.priority = priority; \
+	oh.callerid = callerid; \
+	oh.variable = variable; \
+	oh.account = account; \
+} 
+
+struct outgoing_helper {
+	char *context;
+	char *exten;
+	int priority;
+	char *callerid;
+	char *variable;
+	char *account;
 };
 
 #define AST_CDR_TRANSFER	(1 << 0)
@@ -220,6 +316,13 @@ struct ast_channel {
 #define AST_ADSI_UNAVAILABLE	(2)
 #define AST_ADSI_OFFHOOKONLY	(3)
 
+#define AST_SOFTHANGUP_DEV			(1 << 0)	/* Soft hangup by device */
+#define AST_SOFTHANGUP_ASYNCGOTO	(1 << 1)	/* Soft hangup for async goto */
+#define AST_SOFTHANGUP_SHUTDOWN		(1 << 2)
+#define AST_SOFTHANGUP_TIMEOUT		(1 << 3)
+#define AST_SOFTHANGUP_APPUNLOAD	(1 << 4)
+#define AST_SOFTHANGUP_EXPLICIT		(1 << 5)
+
 /* Bits 0-15 of state are reserved for the state (up/down) of the line */
 /*! Channel is down and available */
 #define AST_STATE_DOWN		0		
@@ -237,11 +340,28 @@ struct ast_channel {
 #define AST_STATE_UP		6		
 /*! Line is busy */
 #define AST_STATE_BUSY  	7		
+/*! Digits (or equivalent) have been dialed while offhook */
+#define AST_STATE_DIALING_OFFHOOK	8
+/*! Channel has detected an incoming call and is waiting for ring */
+#define AST_STATE_PRERING       9
 
 /* Bits 16-32 of state are reserved for flags */
 /*! Do not transmit voice data */
 #define AST_STATE_MUTE		(1 << 16)	
 
+/*! Device is valid but channel didn't know state */
+#define AST_DEVICE_UNKNOWN	0
+/*! Device is not used */
+#define AST_DEVICE_NOT_INUSE	1
+/*! Device is in use */
+#define AST_DEVICE_INUSE	2
+/*! Device is busy */
+#define AST_DEVICE_BUSY		3
+/*! Device is invalid */
+#define AST_DEVICE_INVALID	4
+/*! Device is unavailable */
+#define AST_DEVICE_UNAVAILABLE	5
+
 //! Requests a channel
 /*! 
  * \param type type of channel to request
@@ -253,6 +373,42 @@ struct ast_channel {
  */
 struct ast_channel *ast_request(char *type, int format, void *data);
 
+//! Search the Channels by Name
+/*!
+ * \param device like a dialstring
+ * Search the Device in active channels by compare the channelname against 
+ * the devicename. Compared are only the first chars to the first '-' char.
+ * Returns an AST_DEVICE_UNKNOWN if no channel found or
+ * AST_DEVICE_INUSE if a channel is found
+ */
+int ast_parse_device_state(char *device);
+
+//! Asks a channel for device state
+/*!
+ * \param device like a dialstring
+ * Asks a channel for device state, data is  normaly a number from dialstring
+ * used by the low level module
+ * Trys the channel devicestate callback if not supported search in the
+ * active channels list for the device.
+ * Returns an AST_DEVICE_??? state -1 on failure
+ */
+int ast_device_state(char *device);
+
+/*!
+ * \param type type of channel to request
+ * \param format requested channel format
+ * \param data data to pass to the channel requester
+ * \param timeout maximum amount of time to wait for an answer
+ * \param why unsuccessful (if unsuceessful)
+ * Request a channel of a given type, with data as optional information used 
+ * by the low level module and attempt to place a call on it
+ * Returns an ast_channel on success or no answer, NULL on failure.  Check the value of chan->_state
+ * to know if the call was answered or not.
+ */
+struct ast_channel *ast_request_and_dial(char *type, int format, void *data, int timeout, int *reason, char *callerid);
+
+struct ast_channel *__ast_request_and_dial(char *type, int format, void *data, int timeout, int *reason, char *callerid, struct outgoing_helper *oh);
+
 //! Registers a channel
 /*! 
  * \param type type of channel you are registering
@@ -267,6 +423,11 @@ struct ast_channel *ast_request(char *type, int format, void *data);
 int ast_channel_register(char *type, char *description, int capabilities, 
 			struct ast_channel* (*requester)(char *type, int format, void *data));
 
+/* Same like the upper function but with support for devicestate */
+int ast_channel_register_ex(char *type, char *description, int capabilities,
+		struct ast_channel *(*requester)(char *type, int format, void *data),
+		int (*devicestate)(void *data));
+
 //! Unregister a channel class
 /*
  * \param type the character string that corresponds to the channel you wish to unregister
@@ -292,7 +453,8 @@ int ast_hangup(struct ast_channel *chan);
  * safely hangup a channel managed by another thread.
  * Returns 0 regardless
  */
-int ast_softhangup(struct ast_channel *chan);
+int ast_softhangup(struct ast_channel *chan, int cause);
+int ast_softhangup_nolock(struct ast_channel *chan, int cause);
 
 //! Check to see if a channel is needing hang up
 /*! 
@@ -350,6 +512,27 @@ int ast_indicate(struct ast_channel *chan, int condition);
   Returns < 0 on  failure, 0 if nothing ever arrived, and the # of ms remaining otherwise */
 int ast_waitfor(struct ast_channel *chan, int ms);
 
+//! Wait for a specied amount of time, looking for hangups
+/*!
+ * \param chan channel to wait for
+ * \param ms length of time in milliseconds to sleep
+ * Waits for a specified amount of time, servicing the channel as required.
+ * returns -1 on hangup, otherwise 0.
+ */
+int ast_safe_sleep(struct ast_channel *chan, int ms);
+
+//! Wait for a specied amount of time, looking for hangups and a condition argument
+/*!
+ * \param chan channel to wait for
+ * \param ms length of time in milliseconds to sleep
+ * \param cond a function pointer for testing continue condition
+ * \param data argument to be passed to the condition test function
+ * Waits for a specified amount of time, servicing the channel as required. If cond
+ * returns 0, this function returns.
+ * returns -1 on hangup, otherwise 0.
+ */
+int ast_safe_sleep_conditional(struct ast_channel *chan, int ms, int (*cond)(void*), void *data );
+
 //! Waits for activity on a group of channels
 /*! 
  * \param chan an array of pointers to channels
@@ -376,7 +559,7 @@ int ast_waitfor_n_fd(int *fds, int n, int *ms, int *exception);
 
 
 //! Reads a frame
-/*! 
+/*!
  * \param chan channel to read a frame from
  * Read a frame.  Returns a frame, or NULL on error.  If it returns NULL, you
    best just stop reading frames and assume the channel has been
@@ -392,6 +575,18 @@ struct ast_frame *ast_read(struct ast_channel *chan);
  */
 int ast_write(struct ast_channel *chan, struct ast_frame *frame);
 
+//! Write video frame to a channel
+/*!
+ * \param chan destination channel of the frame
+ * \param frame frame that will be written
+ * This function writes the given frame to the indicated channel.
+ * It returns 1 on success, 0 if not implemented, and -1 on failure.
+ */
+int ast_write_video(struct ast_channel *chan, struct ast_frame *frame);
+
+/* Send empty audio to prime a channel driver */
+int ast_prod(struct ast_channel *chan);
+
 //! Sets read format on channel chan
 /*! 
  * \param chan channel to change
@@ -426,6 +621,9 @@ int ast_sendtext(struct ast_channel *chan, char *text);
  * Read a char of text from a channel
  * Returns 0 on success, -1 on failure
  */
+
+int ast_senddigit(struct ast_channel *chan, char digit);
+
 int ast_recvchar(struct ast_channel *chan, int timeout);
 
 //! Browse channels in use
@@ -433,8 +631,12 @@ int ast_recvchar(struct ast_channel *chan, int timeout);
  * \param prev where you want to start in the channel list
  * Browse the channels currently in use 
  * Returns the next channel in the list, NULL on end.
+ * If it returns a channel, that channel *has been locked*!
  */
-struct ast_channel *ast_channel_walk(struct ast_channel *prev);
+struct ast_channel *ast_channel_walk_locked(struct ast_channel *prev);
+
+//! Get channel by name (locks channel)
+struct ast_channel *ast_get_channel_by_name_locked(char *channame);
 
 //! Waits for a digit
 /*! 
@@ -443,6 +645,10 @@ struct ast_channel *ast_channel_walk(struct ast_channel *prev);
  * Wait for a digit.  Returns <0 on error, 0 on no entry, and the digit on success. */
 char ast_waitfordigit(struct ast_channel *c, int ms);
 
+/* Same as above with audio fd for outputing read audio and ctrlfd to monitor for
+   reading. Returns 1 if ctrlfd becomes available */
+char ast_waitfordigit_full(struct ast_channel *c, int ms, int audiofd, int ctrlfd);
+
 //! Reads multiple digits
 /*! 
  * \param c channel to read from
@@ -454,8 +660,10 @@ char ast_waitfordigit(struct ast_channel *c, int ms);
  * Read in a digit string "s", max length "len", maximum timeout between 
    digits "timeout" (-1 for none), terminated by anything in "enders".  Give them rtimeout
    for the first digit.  Returns 0 on normal return, or 1 on a timeout.  In the case of
-   a timeout, any digits that were read before the timeout will still be available in s.  */
+   a timeout, any digits that were read before the timeout will still be available in s.  
+   RETURNS 2 in full version when ctrlfd is available, NOT 1*/
 int ast_readstring(struct ast_channel *c, char *s, int len, int timeout, int rtimeout, char *enders);
+int ast_readstring_full(struct ast_channel *c, char *s, int len, int timeout, int rtimeout, char *enders, int audiofd, int ctrlfd);
 
 /*! Report DTMF on channel 0 */
 #define AST_BRIDGE_DTMF_CHANNEL_0		(1 << 0)		
@@ -486,7 +694,8 @@ int ast_channel_make_compatible(struct ast_channel *c0, struct ast_channel *c1);
  * \param rc destination channel(?)
  * Bridge two channels (c0 and c1) together.  If an important frame occurs, we return that frame in
    *rf (remember, it could be NULL) and which channel (0 or 1) in rc */
-int ast_channel_bridge(struct ast_channel *c0, struct ast_channel *c1, int flags, struct ast_frame **fo, struct ast_channel **rc);
+//int ast_channel_bridge(struct ast_channel *c0, struct ast_channel *c1, int flags, struct ast_frame **fo, struct ast_channel **rc);
+int ast_channel_bridge(struct ast_channel *c0,struct ast_channel *c1,struct ast_bridge_config *config, struct ast_frame **fo, struct ast_channel **rc);
 
 //! Weird function made for call transfers
 /*! 
@@ -555,15 +764,125 @@ int ast_channel_defer_dtmf(struct ast_channel *chan);
 /*! Undo defer.  ast_read will return any dtmf characters that were queued */
 void ast_channel_undefer_dtmf(struct ast_channel *chan);
 
+/*! Initiate system shutdown -- prevents new channels from being allocated.
+    If "hangup" is non-zero, all existing channels will receive soft
+     hangups */
+void ast_begin_shutdown(int hangup);
+
+/*! Cancels an existing shutdown and returns to normal operation */
+void ast_cancel_shutdown(void);
+
+/*! Returns number of active/allocated channels */
+int ast_active_channels(void);
+
+/*! Returns non-zero if Asterisk is being shut down */
+int ast_shutting_down(void);
+
+/*! Activate a given generator */
+int ast_activate_generator(struct ast_channel *chan, struct ast_generator *gen, void *params);
+
+/*! Deactive an active generator */
+void ast_deactivate_generator(struct ast_channel *chan);
+
+void ast_set_callerid(struct ast_channel *chan, char *callerid, int  anitoo);
+
+/*! Start a tone going */
+int ast_tonepair_start(struct ast_channel *chan, int freq1, int freq2, int duration, int vol);
+/*! Stop a tone from playing */
+void ast_tonepair_stop(struct ast_channel *chan);
+/*! Play a tone pair for a given amount of time */
+int ast_tonepair(struct ast_channel *chan, int freq1, int freq2, int duration, int vol);
+
+/*! Automatically service a channel for us... */
+int ast_autoservice_start(struct ast_channel *chan);
+
+/*! Stop servicing a channel for us...  Returns -1 on error or if channel has been hungup */
+int ast_autoservice_stop(struct ast_channel *chan);
+
+/* If built with zaptel optimizations, force a scheduled expiration on the
+   timer fd, at which point we call the callback function / data */
+int ast_settimeout(struct ast_channel *c, int samples, int (*func)(void *data), void *data);
+
+/* Transfer a channel (if supported).  Returns -1 on error, 0 if not supported
+   and 1 if supported and requested */
+int ast_transfer(struct ast_channel *chan, char *dest);
+
+int ast_do_masquerade(struct ast_channel *chan);
+
+/* Misc. functions below */
+
+/* Helper function for migrating select to poll */
+static inline int ast_fdisset(struct pollfd *pfds, int fd, int max, int *start)
+{
+	int x;
+	for (x=start ? *start : 0;x<max;x++)
+		if (pfds[x].fd == fd) {
+			if (start) {
+				if (x==*start)
+					(*start)++;
+			}
+			return pfds[x].revents;
+		}
+	return 0;
+}
+
+//! Waits for activity on a group of channels
+/*! 
+ * \param nfds the maximum number of file descriptors in the sets
+ * \param rfds file descriptors to check for read availability
+ * \param wfds file descriptors to check for write availability
+ * \param efds file descriptors to check for exceptions (OOB data)
+ * \param tvp timeout while waiting for events
+ * This is the same as a standard select(), except it guarantees the
+ * behaviour where the passed struct timeval is updated with how much
+ * time was not slept while waiting for the specified events
+ */
+static inline int ast_select(int nfds, fd_set *rfds, fd_set *wfds, fd_set *efds, struct timeval *tvp)
+{
+#ifdef __linux__
+	return select(nfds, rfds, wfds, efds, tvp);
+#else
+	if (tvp) {
+		struct timeval tv, tvstart, tvend, tvlen;
+		int res;
+
+		tv = *tvp;
+		gettimeofday(&tvstart, NULL);
+		res = select(nfds, rfds, wfds, efds, tvp);
+		gettimeofday(&tvend, NULL);
+		timersub(&tvend, &tvstart, &tvlen);
+		timersub(&tv, &tvlen, tvp);
+		if (tvp->tv_sec < 0 || (tvp->tv_sec == 0 && tvp->tv_usec < 0)) {
+			tvp->tv_sec = 0;
+			tvp->tv_usec = 0;
+		}
+		return res;
+	}
+	else
+		return select(nfds, rfds, wfds, efds, NULL);
+#endif
+}
+
+#if !defined(ast_strdupa) && defined(__GNUC__)
+# define ast_strdupa(s)									\
+  (__extension__										\
+    ({													\
+      __const char *__old = (s);						\
+      size_t __len = strlen (__old) + 1;				\
+      char *__new = (char *) __builtin_alloca (__len);	\
+      (char *) memcpy (__new, __old, __len);			\
+    }))
+#endif
+
 #ifdef DO_CRASH
-#define CRASH do { *((int *)0) = 0; } while(0)
+#define CRASH do { fprintf(stderr, "!! Forcing immediate crash a-la abort !!\n"); *((int *)0) = 0; } while(0)
 #else
 #define CRASH do { } while(0)
 #endif
 
 #define CHECK_BLOCKING(c) { 	 \
 							if ((c)->blocking) {\
-								ast_log(LOG_WARNING, "Thread %ld Blocking '%s', already blocked by thread %ld in procedure %s\n", pthread_self(), (c)->name, (c)->blocker, (c)->blockproc); \
+								ast_log(LOG_WARNING, "Thread %ld Blocking '%s', already blocked by thread %ld in procedure %s\n", (long) pthread_self(), (c)->name, (long) (c)->blocker, (c)->blockproc); \
 								CRASH; \
 							} else { \
 								(c)->blocker = pthread_self(); \
@@ -571,6 +890,8 @@ void ast_channel_undefer_dtmf(struct ast_channel *chan);
 									c->blocking = -1; \
 									} }
 
+extern unsigned int ast_get_group(char *s);
+
 #if defined(__cplusplus) || defined(c_plusplus)
 }
 #endif
