@@ -14,7 +14,7 @@
 #include <asterisk/module.h>
 #include <asterisk/channel.h>
 #include <asterisk/logger.h>
-#include <pthread.h>
+#include <asterisk/lock.h>
 #include "pbx_kdeconsole.h"
 
 static char *dtext = "KDE Console Monitor";
