@@ -103,6 +103,7 @@ struct parkeduser {
 	int priority;
 	int parkingtime;
 	int notquiteyet;
+	unsigned char moh_trys;
 	struct parkeduser *next;
 };
 
@@ -635,6 +636,11 @@ static void *do_parking_thread(void *ignore)
 						} else {
 							/* XXX Maybe we could do something with packets, like dial "0" for operator or something XXX */
 							ast_frfree(f);
+							if (pu->moh_trys < 3 && !pu->chan->generatordata) {
+								ast_log(LOG_DEBUG, "MOH on parked call stopped by outside source.  Restarting\n");
+								ast_moh_start(pu->chan, NULL);
+								pu->moh_trys++;
+							}
 							goto std;	/* XXX Ick: jumping into an else statement??? XXX */
 						}
 					}
